#include <Arduino.h>
#include <WiFi.h>
#include <string.h>

#include "Config.h"
#include "Dictionary.h"
#include "Display.h"
#include "WiFiMeter.h"
#include "TLV.h"

#include "WiFiMgr.h"


#define MAX_WAIT	10000
#define CONN_WAIT	250


static TLV::Record	ssidrec;
static TLV::Record	wpakeyrec;


bool
SetupWiFi()
{
	WiFi.mode(WIFI_STA);
	return true;
}


static bool
tryConnect(const char *ssid, const char *wpakey)
{
	size_t		 waitedFor = 0;

	Display::Print("CONNECT ");
	Display::Println(ssid);
	WiFi.begin(ssid, wpakey);
	while ((WiFi.status() != WL_CONNECTED) && (waitedFor < MAX_WAIT)) {
		waitedFor += CONN_WAIT;
		if ((waitedFor % 1000) == 0) {
			Display::Print(".");
		}
		delay(250);
	}

	if (WiFi.status() == WL_CONNECTED) {
		Display::Clear(true);
		Display::Print("SSID: ");
		Display::Println(ssid);
		Display::Print("IP: ");
		Display::Println(WiFi.localIP().toString().c_str());

		Config::SetLastSSID(ssid);

		return true;
	}

	Display::Println("CARRIER SIGNAL LOST");
	return false;
}


bool
TryConnect(Dictionary &pb)
{
	uint8_t		*cursor;

	if ((ssidrec.Len = Config::LastSSID(ssidrec.Val, 32)) != 0) {
		Display::Print("LAST SSID: ");
		Display::Println(ssidrec.Val);
		if (pb.Lookup(ssidrec.Val, ssidrec.Len, wpakeyrec)) {
			if (tryConnect((const char *)ssidrec.Val, (const char *)wpakeyrec.Val)) {
				return true;
			}
		} else {
			Display::Println("WPAKEY NOT FOUND");
		}
	}

	while ((cursor != NULL) && (cursor[0] != TAG_EMPTY)) {
		TLV::ReadFromMemory(ssidrec, cursor);
		TLV::SkipRecord(ssidrec, cursor);
		
		TLV::ReadFromMemory(wpakeyrec, cursor);
		TLV::SkipRecord(wpakeyrec, cursor);

		if (tryConnect(ssidrec.Val, wpakeyrec.Val)) {
			return true;
		}
	}

	return false;
}


static bool
tryConnect(Dictionary &pb, int network)
{
	const char	*ssid = WiFi.SSID(network).c_str();
	size_t		 ssidLen = strnlen(ssid, TLV_MAX_LEN);
	TLV::Record	 password;

	if (ssidLen == 0) {
		return false;
	}

	Display::Print("MODEM: CHECK ");
	Display::Print(ssid);
	Display::Print(": ");

	if (!pb.Lookup(ssid, uint8_t(ssidLen), password)) {
		Display::Println("NOT FOUND");
		return false;
	}

	Display::Println("CONNECTING");
	return tryConnect(ssid, password.Val);
}

bool
Autoconnect(Dictionary &pb)
{
	return Autoconnect(pb, true);
}


bool
Autoconnect(Dictionary &pb, bool reset)
{
	int	networkCount = 0;
	int	network = 0;

	Display::Println("MODEM: TRY AUTOCONNECT");

	if (reset) {
		Display::Println("RADIO RESET");
		SetupWiFi();
		WiFi.disconnect();
		delay(1000);
	} else {
		Display::Println("NO RESET");
	}

	networkCount = WiFi.scanNetworks();
	for (network = 0; network < networkCount; network++) {
		if (tryConnect(pb, network)) {
			return true;
		}
	}

	Display::Println("NO CARRIER");
	return false;
}


bool
Reconnect(Dictionary &pb)
{
	if (!tryConnect(ssidrec.Val, wpakeyrec.Val)) {
		return TryConnect(pb);
	}

	return true;
}


void
WiFiRefresh(Dictionary &pb)
{
	Meter::Update();

	if (WiFi.status() == WL_CONNECTED) {
		return;
	}

	Display::Println("CARRIER SIGNAL LOST: ATC");
	Reconnect(pb);
}
